@@ -58,8 +58,9 @@ class Query_Info {
 	enum MYSQL_COM_QUERY_command MyComQueryCmd;
 	bool bool_is_select_NOT_for_update;
 	bool bool_is_select_NOT_for_update_computed;
-	bool have_affected_rows;
+	bool have_affected_rows; // if affected rows is set, last_insert_id is set too
 	uint64_t affected_rows;
+	uint64_t last_insert_id;
 	uint64_t rows_sent;
 	uint64_t waiting_since;
 	std::string show_warnings_prev_query_digest;
@@ -372,9 +373,10 @@ public:
 	unsigned long id;
 	int kill_type;
 	unsigned int hid;
+	int use_ssl;
 
-	KillArgs(char* u, char* p, char* h, unsigned int P, unsigned int _hdi, unsigned long i, int kt, MySQL_Thread* _mt);
-	KillArgs(char *u, char *p, char *h, unsigned int P, unsigned int _hdi, unsigned long i, int kt, MySQL_Thread *_mt, char *ip);
+	KillArgs(char* u, char* p, char* h, unsigned int P, unsigned int _hid, unsigned long i, int kt, int _use_ssl, MySQL_Thread* _mt);
+	KillArgs(char *u, char *p, char *h, unsigned int P, unsigned int _hid, unsigned long i, int kt, int _use_ssl, MySQL_Thread* _mt, char *ip);
 	~KillArgs();
 	const char* get_host_address() const;
 
