@@ -5,6 +5,8 @@
 //#include "thread.h"
 //#include "wqueue.h"
 #include <vector>
+#include <map>
+#include <mutex>
 
 #define STATSDB_SQLITE_TABLE_MYSQL_CONNECTIONS_V1_4 "CREATE TABLE mysql_connections (timestamp INT NOT NULL, Client_Connections_aborted INT NOT NULL, Client_Connections_connected INT NOT NULL, Client_Connections_created INT NOT NULL, Server_Connections_aborted INT NOT NULL, Server_Connections_connected INT NOT NULL, Server_Connections_created INT NOT NULL, ConnPool_get_conn_failure INT NOT NULL, ConnPool_get_conn_immediate INT NOT NULL, ConnPool_get_conn_success INT NOT NULL, Questions INT NOT NULL, Slow_queries INT NOT NULL, PRIMARY KEY (timestamp))"
 
@@ -12,15 +14,18 @@
 
 #define STATSDB_SQLITE_TABLE_MYSQL_CONNECTIONS STATSDB_SQLITE_TABLE_MYSQL_CONNECTIONS_V2_0
 
-#define STATSDB_SQLITE_TABLE_HISTORY_MYSQL_STATUS_VARIABLES_V2_0_10 "CREATE TABLE history_mysql_status_variables (timestamp INT NOT NULL , variable_name VARCHAR NOT NULL , variable_value VARCHAR NOT NULL , PRIMARY KEY (timestamp, variable_name))"
+#define STATSDB_SQLITE_TABLE_HISTORY_MYSQL_STATUS_VARIABLES_V2_4_0 "CREATE TABLE history_mysql_status_variables (timestamp INT NOT NULL , variable_id INT NOT NULL , variable_value VARCHAR NOT NULL , PRIMARY KEY (timestamp, variable_id))"
 
-#define STATSDB_SQLITE_TABLE_HISTORY_MYSQL_STATUS_VARIABLES STATSDB_SQLITE_TABLE_HISTORY_MYSQL_STATUS_VARIABLES_V2_0_10
+#define STATSDB_SQLITE_TABLE_HISTORY_MYSQL_STATUS_VARIABLES STATSDB_SQLITE_TABLE_HISTORY_MYSQL_STATUS_VARIABLES_V2_4_0
+
+#define STATSDB_SQLITE_TABLE_HISTORY_MYSQL_STATUS_VARIABLES_LOOKUP_V2_4_0 "CREATE TABLE  history_mysql_status_variables_lookup (variable_id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL, variable_name VARCHAR NOT NULL, UNIQUE (variable_name))"
+
+#define STATSDB_SQLITE_TABLE_HISTORY_MYSQL_STATUS_VARIABLES_LOOKUP STATSDB_SQLITE_TABLE_HISTORY_MYSQL_STATUS_VARIABLES_LOOKUP_V2_4_0
 
 #define STATSDB_SQLITE_TABLE_HISTORY_STATS_MYSQL_CONNECTION_POOL_V2_0_10 "CREATE TABLE history_stats_mysql_connection_pool (timestamp INT NOT NULL , hostgroup INT , srv_host VARCHAR , srv_port INT , status VARCHAR , ConnUsed INT , ConnFree INT , ConnOK INT , ConnERR INT , MaxConnUsed INT , Queries INT , Queries_GTID_sync INT , Bytes_data_sent INT , Bytes_data_recv INT , Latency_us INT, PRIMARY KEY (timestamp, hostgroup , srv_host , srv_port))"
 
 #define STATSDB_SQLITE_TABLE_HISTORY_STATS_MYSQL_CONNECTION_POOL STATSDB_SQLITE_TABLE_HISTORY_STATS_MYSQL_CONNECTION_POOL_V2_0_10
 
-
 #define STATSDB_SQLITE_TABLE_MYHGM_CONNECTIONS_V2_0  "CREATE TABLE myhgm_connections (timestamp INT NOT NULL, MyHGM_myconnpoll_destroy INT NOT NULL, MyHGM_myconnpoll_get INT NOT NULL, MyHGM_myconnpoll_get_ok INT NOT NULL, MyHGM_myconnpoll_push INT NOT NULL, MyHGM_myconnpoll_reset INT NOT NULL, PRIMARY KEY (timestamp))"
 
 #define STATSDB_SQLITE_TABLE_MYHGM_CONNECTIONS STATSDB_SQLITE_TABLE_MYHGM_CONNECTIONS_V2_0
@@ -119,6 +124,7 @@ class ProxySQL_Statistics {
 	void MySQL_Threads_Handler_sets(SQLite3_result *);
 	void MyHGM_Handler_sets(SQLite3_result *, SQLite3_result *);
 	void system_cpu_sets();
+
 #ifndef NOJEM
 	void system_memory_sets();
 #endif
@@ -131,6 +137,28 @@ class ProxySQL_Statistics {
 #endif
 	SQLite3_result * get_MySQL_Query_Cache_metrics(int interval);
 	void disk_upgrade_mysql_connections();
+
+	/** 
+	 * @brief Retreives the variable id mapped to the provided variable name associated in the history_mysql_variables_lookup table.
+	 * 
+	 * If the variable_name does not have an associaed variable_id assigned in the map, a new variable_id is created for it by loading or creating it using the lookup table.
+	 * This then updates the map if necessary to match the lookup table for the given name.
+	 * 
+	 * @param variable_name The string variable name identifier
+	 * @return Integer variable id for the given variable name
+	 */
+	int64_t get_variable_id_for_name(const std::string & variable_name);
+
+	/** @brief If the variable_name_id_map is empty, then load its contents from all of the history_variables_lookup table records */
+	void load_variable_name_id_map_if_empty();
+
+	/** @return True if the given variable_name is registered in the variable_name_id_map. */
+	bool knows_variable_name(const std::string & variable_name) const;
+
+	private:
+	/** @brief Map with the key being the variable_name and the value being the variable_id, used for history_mysql_variables data. Matches the history_mysql_variables_lookup. */
+	std::map<std::string, int64_t> variable_name_id_map;
+	std::mutex mu;
 };
 
 #endif /* CLASS_PROXYSQL_STATISTICS_H */
