@@ -58,7 +58,8 @@
 #include "proxysql_structs.h"
 #include "proxysql_debug.h"
 #include "proxysql_macros.h"
-
+#include "proxysql_coredump.h"
+#include "proxysql_sslkeylog.h"
 #include "jemalloc.h"
 
 #ifndef NOJEM
@@ -87,7 +88,7 @@
 #else
 #define DEB ""
 #endif /* DEBUG */
-#define PROXYSQL_VERSION	"2.5.1-percona-1.1" DEB
+#define PROXYSQL_VERSION	"2.5.4-percona-1.1" DEB
 #define PROXYSQL_CODENAME	"Truls"
 
 #ifndef PROXYSQL_FUNC_DEFS
