@@ -5,6 +5,7 @@
 #include <prometheus/counter.h>
 #include <prometheus/gauge.h>
 
+#include "query_processor.h"
 #include "proxy_defines.h"
 #include "proxysql.h"
 #include "cpp.h"
@@ -212,6 +213,9 @@ class ProxySQL_Admin {
 #ifdef DEBUG
 		bool debug;
 #endif /* DEBUG */
+		int coredump_generation_interval_ms;
+		int coredump_generation_threshold;
+		char* ssl_keylog_file;
 	} variables;
 
 	unsigned long long last_p_memory_metrics_ts;
@@ -238,6 +242,10 @@ class ProxySQL_Admin {
 	void flush_debug_filters_database_to_runtime(SQLite3DB *db);
 #endif /* DEBUG */
 
+	// Coredump Filters
+	void dump_coredump_filter_values_table();
+	bool flush_coredump_filters_database_to_runtime(SQLite3DB *db);
+
 //	void __insert_or_ignore_maintable_select_disktable(); // commented in 2.3
 	void __insert_or_replace_maintable_select_disktable();
 //	void __delete_disktable(); // commented in 2.3 , unused
@@ -275,8 +283,8 @@ class ProxySQL_Admin {
 	void flush_mysql_variables___database_to_runtime(SQLite3DB *db, bool replace, const std::string& checksum = "", const time_t epoch = 0);
 
 	char **get_variables_list();
-	bool set_variable(char *name, char *value);
-	void flush_admin_variables___database_to_runtime(SQLite3DB *db, bool replace, const std::string& checksum = "", const time_t epoch = 0);
+	bool set_variable(char *name, char *value, bool lock = true);
+	void flush_admin_variables___database_to_runtime(SQLite3DB *db, bool replace, const std::string& checksum = "", const time_t epoch = 0, bool lock = true);
 	void flush_admin_variables___runtime_to_database(SQLite3DB *db, bool replace, bool del, bool onlyifempty, bool runtime=false);
 	void disk_upgrade_mysql_query_rules();
 	void disk_upgrade_mysql_servers();
@@ -309,6 +317,11 @@ class ProxySQL_Admin {
 	void flush_ldap_variables___database_to_runtime(SQLite3DB *db, bool replace, const std::string& checksum = "", const time_t epoch = 0);
 
 	public:
+	/**
+	 * @brief Mutex taken by 'ProxySQL_Admin::admin_session_handler'. It's used prevent multiple
+	 *   ProxySQL_Admin 'sessions' from running in parallel, or for preventing collisions between
+	 *   modules performing operations over the internal SQLite database and 'ProxySQL_Admin' sessions.
+	 */
 	pthread_mutex_t sql_query_global_mutex;
 	struct {
 		void *opt;
@@ -332,6 +345,10 @@ class ProxySQL_Admin {
 	SQLite3DB *configdb; // on disk
 	SQLite3DB *monitordb;	// in memory
 	SQLite3DB *statsdb_disk; // on disk
+#ifdef DEBUG
+	SQLite3DB *debugdb_disk; // on disk for debug
+	int debug_output;
+#endif
 	int pipefd[2];
 	void print_version();
 	bool init();
@@ -345,7 +362,31 @@ class ProxySQL_Admin {
 	void init_mysql_firewall();
 	void init_proxysql_servers();
 	void save_mysql_users_runtime_to_database(bool _runtime);
-	void save_mysql_servers_runtime_to_database(bool);
+	/**
+	 * @brief Save the current MySQL servers reported by 'MySQL_HostGroups_Manager', scanning the
+	 *   current MySQL servers structures for all hostgroups, into either the
+	 *   'main.runtime_mysql_servers' or 'main.mysql_servers' table.
+	 * @param _runtime If true the servers reported by 'MySQL_HostGroups_Manager' are stored into
+	 *   'main.runtime_mysql_servers', otherwise into 'main.runtime_mysql_servers'.
+	 * @details This functions requires the caller to have locked `mysql_servers_wrlock()`, but it
+	 *   doesn't start a transaction as other function that perform several operations over the
+	 *   database. This is because, it's not required doing so, and also because if a transaction
+	 *   was started in the following fashion:
+	 *
+	 *   ```
+	 *   admindb->execute("BEGIN IMMEDIATE");
+	 *   ```
+	 *
+	 *   ProxySQL would lock in 'MySQL_HostGroups_Manager::dump_table_mysql_servers()', or in any
+	 *   other operation from 'MySQL_HostGroups_Manager' that would try to modify the database.
+	 *   Reason being is that trying to modify an attached database during a transaction. Database
+	 *   is only attached for `DEBUG` builds as part of `MySQL_Admin::init()`. Line:
+	 *
+	 *   ```
+	 *   admindb->execute("ATTACH DATABASE 'file:mem_mydb?mode=memory&cache=shared' AS myhgm");
+	 *   ```
+	 */
+	void save_mysql_servers_runtime_to_database(bool _runtime);
 	void admin_shutdown();
 	bool is_command(std::string);
 	void send_MySQL_OK(MySQL_Protocol *myprot, char *msg, int rows=0);
@@ -407,17 +448,25 @@ class ProxySQL_Admin {
 	void load_scheduler_to_runtime();
 	void save_scheduler_runtime_to_database(bool);
 
-	void load_admin_variables_to_runtime(const std::string& checksum = "", const time_t epoch = 0) { flush_admin_variables___database_to_runtime(admindb, true, checksum, epoch); }
+	void load_admin_variables_to_runtime(const std::string& checksum = "", const time_t epoch = 0, bool lock = true) { flush_admin_variables___database_to_runtime(admindb, true, checksum, epoch, lock); }
 	void save_admin_variables_from_runtime() { flush_admin_variables___runtime_to_database(admindb, true, true, false); }
 
 	void load_or_update_global_settings(SQLite3DB *);
 
 	void load_mysql_variables_to_runtime(const std::string& checksum = "", const time_t epoch = 0) { flush_mysql_variables___database_to_runtime(admindb, true, checksum, epoch); }
 	void save_mysql_variables_from_runtime() { flush_mysql_variables___runtime_to_database(admindb, true, true, false); }
-
+	
+	// Coredump filters
+	bool load_coredump_to_runtime() { return flush_coredump_filters_database_to_runtime(admindb); }
+	
 	void p_update_metrics();
 	void stats___mysql_query_rules();
-	void stats___mysql_query_digests(bool reset, bool copy=false);
+	int stats___save_mysql_query_digest_to_sqlite(
+		const bool reset, const bool copy, const SQLite3_result *resultset,
+		const umap_query_digest *digest_umap, const umap_query_digest_text *digest_text_umap
+	);
+	int stats___mysql_query_digests(bool reset, bool copy=false);
+	int stats___mysql_query_digests_v2(bool reset, bool copy, bool use_resultset);
 	//void stats___mysql_query_digests_reset();
 	void stats___mysql_commands_counters();
 	void stats___mysql_processlist();
@@ -505,8 +554,16 @@ class ProxySQL_Admin {
 	void enable_grouprep_testing();
 #endif // TEST_GROUPREP
 
+#ifdef TEST_READONLY
+	void enable_readonly_testing();
+#endif // TEST_READONLY
+
+#ifdef TEST_REPLICATIONLAG
+	void enable_replicationlag_testing();
+#endif // TEST_REPLICATIONLAG
+
 	unsigned int ProxySQL_Test___GenerateRandom_mysql_query_rules_fast_routing(unsigned int, bool);
-	bool ProxySQL_Test___Verify_mysql_query_rules_fast_routing(int *ret1, int *ret2, int cnt, int dual);
+	bool ProxySQL_Test___Verify_mysql_query_rules_fast_routing(int *ret1, int *ret2, int cnt, int dual, int ths, bool lock, bool maps_per_thread);
 	void ProxySQL_Test___MySQL_HostGroups_Manager_generate_many_clusters();
 	unsigned long long ProxySQL_Test___MySQL_HostGroups_Manager_read_only_action();
 #ifdef DEBUG
