@@ -58,15 +58,6 @@
   } while (rc!=SQLITE_DONE);\
 } while (0)
 
-#define SAFE_SQLITE3_STEP2(_stmt) do {\
-	do {\
-	rc=(*proxy_sqlite3_step)(_stmt);\
-		if (rc==SQLITE_LOCKED || rc==SQLITE_BUSY) {\
-			usleep(100);\
-		}\
-	} while (rc==SQLITE_LOCKED || rc==SQLITE_BUSY);\
-} while (0)
-
 #include "clickhouse/client.h"
 
 using namespace clickhouse;
@@ -1371,7 +1362,7 @@ static void *child_mysql(void *arg) {
 		}
 		myds->revents=fds[0].revents;
 		int rb = 0;
-		rb - myds->read_from_net();
+		rb = myds->read_from_net();
 		if (myds->net_failure) goto __exit_child_mysql;
 		myds->read_pkts();
 		if (myds->encrypted == true) {
@@ -1483,8 +1474,11 @@ __end_while_pool:
                                 } else {
                                         c_split_2(sn, ":" , &add, &port);
                                 }
-
+#ifdef SO_REUSEPORT
+				int s = ( atoi(port) ? listen_on_port(add, atoi(port), 128, true) : listen_on_unix(add, 128));
+#else
 				int s = ( atoi(port) ? listen_on_port(add, atoi(port), 128) : listen_on_unix(add, 128));
+#endif // SO_REUSEPORT
 				if (s>0) { fds[nfds].fd=s; fds[nfds].events=POLLIN; fds[nfds].revents=0; callback_func[nfds]=0; socket_names[nfds]=strdup(sn); nfds++; }
 				if (add) free(add);
 				if (port) free(port);
