@@ -77,6 +77,22 @@ ProxySQL_GlobalVariables::~ProxySQL_GlobalVariables() {
 		free(ldap_auth_plugin);
 		ldap_auth_plugin = NULL;
 	}
+	/**
+	 * @brief set in_shutdown flag just the member 'checksums_values'.
+	 * @details This is performed to prevent the free() inside the 'ProxySQL_Checksum_Value' destructor for
+	 *  'checksums_values' members. These checksums memory is never freed during ProxySQL execution
+	 *  lifetime (only reused during update operations), and they are concurrently access by multiple threads,
+	 *  including during shutdown phase. Since 'GloVars' (unique instance of this class) is declared global,
+	 *  it's impossible to control de destruction order with respect to the other modules. To avoid invalid
+	 *  memory accesses during shutdown, we avoid calling free() inside the destructor of the members.
+	 */
+	checksums_values.admin_variables.in_shutdown = true;
+	checksums_values.mysql_query_rules.in_shutdown = true;
+	checksums_values.mysql_servers.in_shutdown = true;
+	checksums_values.mysql_users.in_shutdown = true;
+	checksums_values.mysql_variables.in_shutdown = true;
+	checksums_values.ldap_variables.in_shutdown = true;
+	checksums_values.proxysql_servers.in_shutdown = true;
 };
 
 ProxySQL_GlobalVariables::ProxySQL_GlobalVariables() :
@@ -128,9 +144,11 @@ ProxySQL_GlobalVariables::ProxySQL_GlobalVariables() :
 #endif
 
 	global.sqlite3_server=false;
+	global.data_packets_history_size=0;
 #ifdef PROXYSQLCLICKHOUSE
 	global.clickhouse_server=false;
 #endif /* PROXYSQLCLICKHOUSE */
+	global.ssl_keylog_enabled = false;
 	opt=new ez::ezOptionParser();
 	opt->overview="High Performance Advanced Proxy for MySQL";
 	opt->syntax="proxysql [OPTIONS]";
@@ -278,7 +296,9 @@ void ProxySQL_GlobalVariables::process_opts_pre() {
 #ifdef DEBUG
 	init_debug_struct();
 #endif
+	init_coredump_struct();
 
+	proxysql_keylog_init();
 };
 
 void ProxySQL_GlobalVariables::process_opts_post() {
