@@ -10,7 +10,7 @@
 #include <unistd.h>
 #include <sys/wait.h>
 
-#include <mysql.h>
+#include "mysql.h"
 
 #include "tap.h"
 #include "utils.h"
@@ -589,7 +589,7 @@ int wait_for_replication(
 	return result;
 }
 
-MARIADB_CHARSET_INFO * proxysql_find_charset_collate(const char *collatename) {
+MARIADB_CHARSET_INFO * proxysqlTap_find_charset_collate(const char *collatename) {
 	MARIADB_CHARSET_INFO *c = (MARIADB_CHARSET_INFO *)mariadb_compiled_charsets;
 	do {
 		if (!strcasecmp(c->name, collatename)) {
@@ -812,10 +812,10 @@ int get_variable_value(
 		res = EXIT_SUCCESS;
 	}
 
-	mysql_free_result(admin_res);
-
 cleanup:
 
+	mysql_free_result(admin_res);
+
 	return res;
 }
 
@@ -1069,3 +1069,190 @@ int configure_endpoints(
 
 	return EXIT_SUCCESS;
 }
+
+int extract_sqlite3_host_port(MYSQL* admin, std::pair<std::string, int>& host_port) {
+	if (admin == nullptr) { return EINVAL; }
+
+	const char varname[] { "sqliteserver-mysql_ifaces" };
+	string sqlite3_ifaces {};
+
+	// ProxySQL is likely to have been launched without "--sqlite3-server" flag
+	if (get_variable_value(admin, varname, sqlite3_ifaces)) {
+		diag("ProxySQL was launched without '--sqlite3-server' flag");
+		return EXIT_FAILURE;
+	}
+
+	// Extract the correct port to connect to SQLite server
+	std::string::size_type colon_pos = sqlite3_ifaces.find(":");
+	if (colon_pos == std::string::npos) {
+		diag("ProxySQL returned a malformed 'sqliteserver-mysql_ifaces': %s", sqlite3_ifaces.c_str());
+		return EXIT_FAILURE;
+	}
+
+	std::string sqlite3_host { sqlite3_ifaces.substr(0, colon_pos) };
+	std::string sqlite3_port { sqlite3_ifaces.substr(colon_pos + 1) };
+
+	// Check that port has valid conversion
+	char* end_pos = nullptr;
+	int i_sqlite3_port = std::strtol(sqlite3_port.c_str(), &end_pos, 10);
+
+	if (errno == ERANGE || (end_pos != &sqlite3_port.back() + 1)) {
+		diag("ProxySQL returned a invalid port number within 'sqliteserver-mysql_ifaces': %s", sqlite3_ifaces.c_str());
+		return EXIT_FAILURE;
+	}
+
+	host_port = { sqlite3_host, i_sqlite3_port };
+
+	return EXIT_SUCCESS;
+}
+
+std::vector<std::string> split(const std::string& s, char delim) {
+	std::istringstream tokenStream(s);
+	std::vector<std::string> tokens {};
+
+	std::string token {};
+	while (std::getline(tokenStream, token, delim)) {
+		tokens.push_back(token);
+	}
+
+	return tokens;
+}
+
+string get_env(const string& var) {
+	string f_path {};
+
+	char* p_infra_datadir = std::getenv(var.c_str());
+	if (p_infra_datadir != nullptr) {
+		f_path = p_infra_datadir;
+	}
+
+	return f_path;
+}
+
+int open_file_and_seek_end(const string& f_path, std::fstream& f_stream) {
+	const char* c_f_path { f_path.c_str() };
+	f_stream.open(f_path.c_str(), std::fstream::in | std::fstream::out);
+
+	if (!f_stream.is_open() || !f_stream.good()) {
+		diag("Failed to open '%s' file: { path: %s, error: %d }", basename(c_f_path), c_f_path, errno);
+		return EXIT_FAILURE;
+	}
+
+	f_stream.seekg(0, std::ios::end);
+
+	return EXIT_SUCCESS;
+}
+
+std::vector<line_match_t> get_matching_lines(std::fstream& f_stream, const std::string& regex) {
+	std::vector<line_match_t> found_matches {};
+
+	std::string next_line {};
+	std::fstream::pos_type init_pos { f_stream.tellg() };
+
+	while (std::getline(f_stream, next_line)) {
+		std::regex regex_err_line { regex };
+		std::smatch match_results {};
+
+		if (std::regex_search(next_line, match_results, regex_err_line)) {
+			found_matches.push_back({ f_stream.tellg(), next_line, match_results });
+		}
+	}
+
+	if (found_matches.empty() == false) {
+		const std::string& last_match { std::get<LINE_MATCH_T::LINE>(found_matches.back()) };
+		const std::fstream::pos_type last_match_pos { std::get<LINE_MATCH_T::POS>(found_matches.back()) };
+
+		f_stream.clear(f_stream.rdstate() & ~std::ios_base::failbit);
+		f_stream.seekg(last_match_pos);
+	} else {
+		f_stream.clear(f_stream.rdstate() & ~std::ios_base::failbit);
+		f_stream.seekg(init_pos);
+	}
+
+	return found_matches;
+}
+
+const uint32_t USLEEP_SQLITE_LOCKED = 100;
+
+int open_sqlite3_db(const string& f_path, sqlite3** db, int flags) {
+	const char* c_f_path { f_path.c_str() };
+	const char* base_path { basename(c_f_path) };
+	int rc = sqlite3_open_v2(f_path.c_str(), db, flags, NULL);
+
+	if (rc) {
+		const char* err_msg = *db == nullptr ? "Failed to allocate" : sqlite3_errmsg(*db);
+		diag("Failed to open sqlite3 db-file '%s': { path: %s, error: %s }", base_path, c_f_path, err_msg);
+		return EXIT_FAILURE;
+	} else {
+		return EXIT_SUCCESS;
+	}
+}
+
+sq3_res_t sqlite3_execute_stmt(sqlite3* db, const string& query) {
+	int rc = 0;
+	sqlite3_stmt* stmt = NULL;
+	sq3_res_t res {};
+
+	do {
+		rc = sqlite3_prepare_v2(db, query.c_str(), -1, &stmt, 0);
+		if (rc==SQLITE_LOCKED || rc==SQLITE_BUSY) {
+			usleep(USLEEP_SQLITE_LOCKED);
+		}
+	} while (rc==SQLITE_LOCKED || rc==SQLITE_BUSY);
+
+	if (rc != SQLITE_OK) {
+		res = {{}, {}, {}, sqlite3_errmsg(db)};
+		goto cleanup;
+	}
+
+	{
+		// extract a resultset or just evaluate
+		uint32_t cols_count = sqlite3_column_count(stmt);
+
+		if (cols_count == 0) {
+			do {
+				rc = sqlite3_step(stmt);
+				if (rc==SQLITE_LOCKED || rc==SQLITE_BUSY) {
+					usleep(USLEEP_SQLITE_LOCKED);
+				}
+			} while (rc==SQLITE_LOCKED || rc==SQLITE_BUSY);
+
+			if (rc == SQLITE_DONE) {
+				uint32_t affected_rows = sqlite3_changes(db);
+				res = {{}, {}, affected_rows, {}};
+			} else {
+				res = {{}, {}, {}, sqlite3_errmsg(db)};
+				goto cleanup;
+			}
+		} else {
+			vector<sq3_col_def_t> cols_defs {};
+			vector<sq3_row_t> rows {};
+
+			for (uint32_t i = 0; i < cols_count; i++) {
+				cols_defs.push_back(sqlite3_column_name(stmt, i));
+			}
+
+			while ((rc = sqlite3_step(stmt)) == SQLITE_ROW) {
+				sq3_row_t row {};
+
+				for (uint32_t i = 0; i < cols_count; i++) {
+					if (sqlite3_column_type(stmt, i) == SQLITE_NULL) {
+						row.push_back({});
+					} else {
+						row.push_back(reinterpret_cast<const char*>(sqlite3_column_text(stmt, i)));
+					}
+				}
+
+				rows.push_back(row);
+			}
+
+			res = { cols_defs, rows, 0, {} };
+		}
+	}
+
+cleanup:
+	sqlite3_reset(stmt);
+	sqlite3_finalize(stmt);
+
+	return res;
+}
