@@ -57,6 +57,7 @@ std::vector<std::string> queries = {
 	"CHECKSUM MEM MYSQL QUERY RULES",
 	"CHECKSUM MEM MYSQL VARIABLES",
 	"CHECKSUM MEM MYSQL REPLICATION HOSTGROUPS",
+	"CHECKSUM MEM MYSQL HOSTGROUP ATTRIBUTES",
 	"CHECKSUM MEM MYSQL GROUP REPLICATION HOSTGROUPS",
 	"CHECKSUM MEM MYSQL GALERA HOSTGROUPS",
 	"CHECKSUM MEM MYSQL AURORA HOSTGROUPS",
@@ -65,6 +66,7 @@ std::vector<std::string> queries = {
 	"CHECKSUM MEMORY MYSQL QUERY RULES",
 	"CHECKSUM MEMORY MYSQL VARIABLES",
 	"CHECKSUM MEMORY MYSQL REPLICATION HOSTGROUPS",
+	"CHECKSUM MEMORY MYSQL HOSTGROUP ATTRIBUTES",
 	"CHECKSUM MEMORY MYSQL GROUP REPLICATION HOSTGROUPS",
 	"CHECKSUM MEMORY MYSQL GALERA HOSTGROUPS",
 	"CHECKSUM MEMORY MYSQL AURORA HOSTGROUPS",
@@ -73,6 +75,7 @@ std::vector<std::string> queries = {
 	"CHECKSUM MYSQL QUERY RULES",
 	"CHECKSUM MYSQL VARIABLES",
 	"CHECKSUM MYSQL REPLICATION HOSTGROUPS",
+	"CHECKSUM MYSQL HOSTGROUP ATTRIBUTES",
 	"CHECKSUM MYSQL GROUP REPLICATION HOSTGROUPS",
 	"CHECKSUM MYSQL GALERA HOSTGROUPS",
 	"CHECKSUM MYSQL AURORA HOSTGROUPS",
