@@ -129,7 +129,7 @@ int main() {
 	vars["sql_log_bin"]->add(bool_values);
 	vars["sql_safe_updates"] = new variable("sql_safe_updates", true, false, true);
 	vars["sql_safe_updates"]->add(bool_values);
-	vars["sql_big_selects"] = new variable("sql_safe_updates", true, false, true);
+	vars["sql_big_selects"] = new variable("sql_big_selects", true, false, true);
 	vars["sql_big_selects"]->add(bool_values);
 //	vars["wsrep_sync_wait"] = new variable("wsrep_sync_wait", true, false);
 //	vars["wsrep_sync_wait"]->add(bool_values);
@@ -205,6 +205,9 @@ int main() {
 	vars["sort_buffer_size"] = new variable("sort_buffer_size", true, true, false);
 	vars["sort_buffer_size"]->add(int_values, 40123);
 	vars["sort_buffer_size"]->add("18446744073709551615");
+	vars["max_statement_time"] = new variable("max_statement_time", true, true, false);
+	vars["max_statement_time"]->add(int_values, 12012);
+	vars["max_statement_time"]->add("0");
 	vars["max_execution_time"] = new variable("max_execution_time", true, true, false);
 	vars["max_execution_time"]->add(int_values, 12930);
 	vars["max_execution_time"]->add("0");
@@ -228,8 +231,8 @@ int main() {
 	vars["session_track_gtids"]->add("OWN_GTID");
 //	vars["session_track_gtids"]->add("OFF");
 //	vars["session_track_gtids"]->add("ALL_GTID");
-	
-	
+
+
 	vars["optimizer_switch"] = new variable("optimizer_switch", true, false, false);
 	vars["optimizer_switch"]->add(std::vector<std::string> {"'materialization=off'", "`materialization=on`", "\"materialization=off\""});
 	vars["optimizer_switch"]->add(std::vector<std::string> {"'index_merge_union=off'", "`index_merge_union=on`", "\"index_merge_union=off\""});
@@ -249,6 +252,10 @@ int main() {
 	vars["lc_messages"]->add(std::vector<std::string> {"es_ES", "'es_ES'", "`es_ES`", "\"es_ES\""});
 	vars["lc_messages"]->add(std::vector<std::string> {"fr_FR", "'fr_FR'", "`fr_FR`", "\"fr_FR\""});
 
+    vars["log_slow_filter"] = new variable("log_slow_filter", true, false, false);
+	vars["log_slow_filter"]->add(std::vector<std::string> {"'not_using_index'", "`not_using_index`", "\"not_using_index\""});
+	vars["log_slow_filter"]->add(std::vector<std::string> {"'admin,filesort,filesort_on_disk,full_join'", "`admin,filesort,filesort_on_disk,full_join`", "\"admin,filesort,filesort_on_disk,full_join\""});
+
 	vars["time_zone"] = new variable("time_zone", true, false, false);
 	vars["time_zone"]->add(std::vector<std::string> {"'+01:00'", "`+02:15`", "\"+03:30\""});
 	vars["time_zone"]->add(std::vector<std::string> {"'+04:45'", "`+05:00`", "\"+06:10\""});
@@ -277,6 +284,15 @@ int main() {
 
 	vars["wsrep_osu_method"] = new variable("wsrep_osu_method", true, false, false);
 	add_values_and_quotes("wsrep_osu_method", {"TOI","RSU"});
+
+	vars["sql_quote_show_create"] = new variable("sql_quote_show_create", true, false, true);
+	vars["sql_quote_show_create"]->add(bool_values);
+
+	vars["sql_require_primary_key"] = new variable("sql_require_primary_key", true, false, true);
+	vars["sql_require_primary_key"]->add(bool_values);
+
+	vars["sql_generate_invisible_primary_key"] = new variable("sql_generate_invisible_primary_key", true, false, true);
+	vars["sql_generate_invisible_primary_key"]->add(bool_values);
 /*
 example:
 "SET sql_mode='NO_ENGINE_SUBSTITUTION', sql_select_limit=3030, session_track_gtids=OWN_GTID; SET max_join_size=10000; ", "{'sql_mode':'NO_ENGINE_SUBSTITUTION','sql_select_limit':'3030', 'max_join_size':'10000', 'session_track_gtids':'OWN_GTID'}"
