@@ -172,6 +172,12 @@ void * my_conn_thread(void *arg) {
 				} else {
 					vars[el.key()] = el.value();
 				}
+			} else if (el.key() == "max_execution_time") {
+				if (is_mariadb) {
+					vars["max_statement_time"] = el.value();
+				} else {
+					vars[el.key()] = el.value();
+				}
 			}
 			else {
 				vars[el.key()] = el.value();
